/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include <etherxd.h>

char *etherx__remote = NULL;
char *etherx__secret = NULL;
thread etherx__daemon = NULL;
ehandler etherx__handlers = NULL;

void etherx_init(char *secret, char *host)
{
    if(host != NULL)
        etherx__remote = strdup(host);
    etherx__secret = strdup(secret);
}

ehandler ehandler_new(etherx_handler h, char *host, char *namespace)
{
    pool p;
    ehandler e;

    log_debug(ZONE,"ehandler_new(%s,%s)",host,namespace);

    /* init new ehandler */
    p = pool_heap(1*1024);
    e = pmalloc(p, sizeof(_ehandler));
    e->host = pstrdup(p, host);
    e->namespace = pstrdup(p, namespace);
    e->p = p;
    e->h = h;
    e->t = NULL;
    e->attempts = 0;

    /* we're not a true handler, just a stub */
    if(h == NULL)
        return e;

    /* keep a master list of all the handlers that want to receive data */
    if(host != NULL)
        etherx__handlers = ehandler_list_add(etherx__handlers,e);

    /* this process is the daemon, we don't need to do anything else */
    if(etherx__daemon != NULL)
        return e;

    /* try connecting a new xmlstream */
    etherxs_connect(e);

    log_debug(ZONE,"spun off connection attempt");

    return e;
}

void ehandler_send(ehandler eh, xmlnode x, char *to)
{
    epacket ep;

    if(eh == NULL || x == NULL || to == NULL)
    {
        log_warn("etherx","received illegal outgoing data");
        return;
    }

    ep = epacket_new(x, to, eh->host, eh->namespace);

    /* running daemon */
    if(etherx__daemon)
    {
        log_debug(ZONE,"ehandler_send: daemon %s",xmlnode2str(x));
        etherxd_send(ep);
        return;
    }

    /* connected socket */
    if(eh->t != NULL)
    {
        log_debug(ZONE,"ehandler_send: socket %s",xmlnode2str(x));
        xmlstream_write(eh->t, epacket_serialize(ep));
        return;
    }

    /* if we get here it's because the socket isn't connected, bounce back to the handler */
    epacket_error(ep,"delivery failure");
    (*eh->h)(ep);
}

