/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include <etherx.h>
#include "config.h"

int tlisten_default = 10;

thread tlisten_new(u_short port, char *ip, tlisten_onConnect evt, void *evt_data)
{
    thread t = NULL;
    tlisten tl = NULL;
    int s;
    pth_attr_t attr;

    log_debug(ZONE,"tlisten_start: %s",ip);

    s = make_netsocket(port, ip, NETSOCKET_SERVER);
    if(s < 0)
        return NULL;

    if(listen(s, tlisten_default) < 0)
        return NULL;

    t = thread_new();

    tl = pmalloc(t->p, sizeof(_tlisten));
    tl->s = s;
    tl->connect_evt = evt;
    tl->connect_evt_data = evt_data;
    tl->port = port;
    tl->ip = j_strdup(ip);

    t->data = (void *)tl;

    attr = pth_attr_new();
    pth_attr_set(attr, PTH_ATTR_JOINABLE, FALSE);
    pth_attr_set(attr, PTH_ATTR_STACK_SIZE, etherx_stack_default);
    t->id = pth_spawn(attr, _tlisten_main, (void *)t);
    pth_attr_destroy(attr);

    return t;
}

void _tlisten_cleanup(void *arg)
{
    thread t = (thread)arg;
    tlisten tl = (tlisten)(t->data);

    log_debug(ZONE,"tlisten_cleanup");

    close(tl->s);
    thread_free(t);
}

void _tlisten_cleanup_rate(void *arg)
{
    rate_free((rlimit)arg);
}

void *_tlisten_main(void *arg)
{
    /* the guts of the thread, main loop */
    thread t = (thread)arg;
    tlisten tl = (tlisten)(t->data);
    struct sockaddr_in sa;
    size_t sa_size = sizeof(sa);
    rlimit all_r, ip_r;
    int new;

    log_debug(ZONE,"tlisten_main: %s","startup");

    pth_cleanup_push(_tlisten_cleanup, (void *)(t));

    all_r = rate_new(10, 150); /* 150 connects per 10 seconds */
    ip_r = rate_new(5, 25); /* 25 connects per 5 seconds for any single ip */
    pool_cleanup(t->p, _tlisten_cleanup_rate, (void *)(all_r));
    pool_cleanup(t->p, _tlisten_cleanup_rate, (void *)(ip_r));

    while(1)
    {
        new = pth_accept(tl->s, (struct sockaddr *) &sa, (int *) &sa_size);
        if(new < 0)
            break;

#ifndef UNRATED
        if(rate_check(ip_r,inet_ntoa(sa.sin_addr),1))
        {
            log_warn("etherx","%s is being connection rate limited",inet_ntoa(sa.sin_addr));
            close(new);
            continue;
        }
        if(rate_check(all_r,"a",1))
        {
            log_warn("etherx","possible mass-connect DoS attack, slowing accepts");
            pth_sleep(1);
        }
#endif

        log_debug(ZONE,"tlisten_main: new socket accepted");
        (tl->connect_evt)(t, new, sa);
    }

    return NULL;
}

